#!/bin/sh

if [ -z "$1" ]; then
  echo "UPGRADE FAIL: Empty version info specified in the image, cannot check if the new firmware supports this hardware"
  echo "UPGRADE FAIL: Empty version info specified in the image, cannot check if the new firmware supports this hardware" > /dev/kmsg
  exit 0;
fi

new_version=$1
min_version="0.0.0.0";
required_sw_version="0.0.0.0";

case "$new_version" in 
  *local*)  exit 1;; # always allow local builds
esac

cur_board=`factory board`
current_hw=`factory hw`

case "$cur_board"  in
    "WOP-2ac-LR5:rev.B")    min_version="1.4.2.1";;
    "WOP-2ac-LR5 SYNC")     min_version="1.7.0.130";;
    "WOP-2ac-LR2 SYNC")     min_version="1.7.0.274";;
    "WEP-1L")
        accept_hw="2v3"
        required_sw_version="1.2.4.2";
        ;;
    "WES-1L")
        accept_hw="2v3"
        required_sw_version="1.2.4.2";
        ;;
    "WEP-2L")
        accept_hw="3v0"
        required_sw_version="1.2.4.2";
        ;;
    "WOP-2L")
        accept_hw="2v0"
        required_sw_version="1.2.4.2";
        ;;
esac

# replace ' build ' with '.'
new_version=$(echo "$new_version" | sed "s/[[:space:]]\?build[[:space:]]\?/\./")
min_version=$(echo "$min_version" | sed "s/[[:space:]]\?build[[:space:]]\?/\./")

if [[ "$required_sw_version" != "0.0.0.0" ]]
then
    acceptable_hw_major=$(echo $accept_hw|cut -d'v' -f 1)
    current_hw_major=$(echo $current_hw|cut -d'v' -f 1)
    acceptable_hw_minor=$(echo $accept_hw|cut -d'v' -f 2)
    current_hw_minor=$(echo $current_hw|cut -d'v' -f 2)

    if [ $current_hw_major -gt $acceptable_hw_major ] || ([ $current_hw_major -eq $acceptable_hw_major ] && [ $current_hw_minor -ge $acceptable_hw_minor ])
    then
        min_version=$required_sw_version
    else
        exit 1;
    fi
fi

for i in 1 2 3 4
do
    new=$(echo $new_version|cut -d'.' -f $i|cut -d'-' -f 1)
    min=$(echo $min_version|cut -d'.' -f $i)
    case "$new" in
        ''|*[!0-9]*)  new=0;; # replace empty and invalid numbers with 0
    esac
    case "$min" in
        ''|*[!0-9]*)  min=0;; # replace empty and invalid numbers with 0
    esac
    if [ "$new" -gt "$min" ]; then
        break
    elif [ "$new" -lt "$min" ]; then
        echo "UPGRADE FAIL: Version of the new firmware is less than the minimum version ${min_version} that supports this hardware"
        echo "UPGRADE FAIL: Version of the new firmware is less than the minimum version ${min_version} that supports this hardware" > /dev/kmsg
        exit 0;
    fi
done

exit 1;
