#!/bin/sh

echo 2 > /proc/sys/vm/overcommit_memory
echo 100 > /proc/sys/vm/overcommit_ratio

echo "Start firmware upgrading"
echo "Start firmware upgrading" > /dev/kmsg

if [ -f /tmp/fw/utils/upgrade-bin ]; then
    FW_PATH="/tmp/fw"
else
    if [ -f /tmp/auto_fw_upgrade/utils/upgrade-bin ]; then
        FW_PATH="/tmp/auto_fw_upgrade"
    else
        if [ -f /tmp/dhcp_fw_upgrade/utils/upgrade-bin ]; then
            FW_PATH="/tmp/dhcp_fw_upgrade"
        else
            FW_PATH="/tmp/fw_tr069"
        fi
    fi
fi

#some validation
if [ -f "./check_compability.sh" ]; then
    ./check_compability.sh "./fmw_board_type"
    retval=$?
    if [ "$retval" -eq "0" ]; then
        exit 1;
    fi
else
    if [ -f "/bin/check_compability.sh" ]; then
        /bin/check_compability.sh "./fmw_board_type"
        retval=$?
        if [ "$retval" -eq "0" ]; then
            exit 1;
        fi
    fi
fi

# check if the new firmware version supports this hardware
if [ -f $FW_PATH/version ]; then
    NEW_VERSION=$(cat $FW_PATH/version)
    output=`./check_version.sh "${NEW_VERSION}")`
    retval=$?
    if [ "$retval" -eq "0" ]; then
        echo $output
        echo $output > /dev/kmsg
        exit 1
    fi
else
    echo "UPGRADE FAIL: No version info in the image, cannot check if the new firmware supports this hardware"
    echo "UPGRADE FAIL: No version info in the image, cannot check if the new firmware supports this hardware" > /dev/kmsg
    exit 1
fi

md5_cur=""

if [ -f ./linux.bin ]; then
    md5_cur=`busybox md5sum ./linux.bin | cut -d' ' -f 1`
else
    md5_cur=`tar -xOf $1 linux.bin | busybox md5sum | cut -d' ' -f 1`
fi

md5_img=`cat md5_linux`

if [ "$md5_cur" != "$md5_img" ]; then
    echo "UPGRADE FAIL: Incorrect CRC";
    exit 1;
fi


if [ -f $FW_PATH/upgrade-bin ]; then
    chmod 777 $FW_PATH/upgrade-bin
fi

cur_major=$(cat /version|cut -d'.' -f 1)
cur_minor=$(cat /version|cut -d'.' -f 2)

echo "Upgrade firmware started"
echo "Upgrade firmware started" > /dev/kmsg

if [ "$DIR_CHANGES" == "down" ]; then
    echo "We are using /bin/upgrade-bin"
    if [ -f ./linux.bin ]; then
        /bin/upgrade-bin ./linux.bin &> /dev/kmsg
    else
        tar -xOf $1 linux.bin | /bin/upgrade-bin &> /dev/kmsg
    fi
else
    echo "We are using $FW_PATH/utils/upgrade-bin"
    if [ -f ./linux.bin ]; then
        $FW_PATH/utils/upgrade-bin ./linux.bin &> /dev/kmsg
    else
        tar -xOf $1 linux.bin | $FW_PATH/utils/upgrade-bin &> /dev/kmsg
    fi
fi
RETVAL=$?

if [ "$RETVAL" != "0" ]; then
    if [ "$RETVAL" = "254" ]; then
	echo "UPGRADE FAIL: upgrade process is already in progress" > /dev/kmsg;
	echo "UPGRADE FAIL: upgrade process is already in progress";
	exit 2;
    fi
    echo "UPGRADE FAIL: Error at read/write operation" > /dev/kmsg;
    echo "UPGRADE FAIL: Error at read/write operation";
    exit 1;
fi

if [ -f /proc/bootbank ]; then
    BANK=$(cat /proc/bootbank)
    if [ -f $FW_PATH/version ]; then
        NEW_VERSION=$(cat $FW_PATH/version)
        if [ "$BANK" = "1" ]; then
            fwctl set 2 "$NEW_VERSION"
        else
            if [ "$BANK" = "2" ]; then
                fwctl set 1 "$NEW_VERSION"
            fi
        fi
    fi
fi

if [ -e "$FW_PATH/defaults" ];then
    echo "UPGRADE : update default config" > /dev/kmsg;
    echo "UPGRADE : update default config";
    cd $FW_PATH
    cat /proc/mtd | grep "mtd2" > /dev/null
    if [ $? == 0 ]; then
        if [ "$DIR_CHANGES" == "down" ]; then
            tar -c defaults | gzip | /bin/save_def_yaml
        else
            tar -c defaults | gzip | $FW_PATH/utils/save_def_yaml
        fi
    else
        tar -c defaults | gzip | $FW_PATH/utils/save_def_yaml_old
    fi
#    tar -c defaults | gzip | $FW_PATH/utils/save_def_yaml
    cd -
fi

echo "UPGRADE SUCCESSFULL" > /dev/kmsg
echo "UPGRADE SUCCESSFULL"
exit 0;
